/******************************************************************************
 *
 * FILE: can.c
 *
 * Description:
 *    CAN communication
 *
 * History:
 *    17.11.2017   PN   Initial version
 *    21.02.2018   MF   Rework for SPC570S50, FlexCAN
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

/* BSP support package */
#include "bsp.h"
/* CAN communication header */
#include "can.h"

/* ============================================================================
 *            GLOBAL VARIABLES
 * ==========================================================================*/

CAN_MESSAGE_OBJECT_t g_can_RxData;
CAN_MESSAGE_OBJECT_t g_can_TxData = {.ID = 0x10, .LENGTH = 8,                \
					.Data[0] = 0, .Data[1] = 1, .Data[2] = 2, .Data[3] = 3,  \
					.Data[4] = 4, .Data[5] = 5, .Data[6] = 6, .Data[7] = 7};
/* ============================================================================
 *            ISR FUNCTIONS
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * FUNCTION: CanIsrHandler()
 *    RX FIFO ISR Handler
 * --------------------------------------------------------------------------*/

void CanIsrHandler(void)
{
	/* Frames available in FIFO */
	if (FLEXCAN_0.IFLAG1.B.BUF5I == 1)
	{
		/* Get ...*/
		/* Length */
		g_can_RxData.LENGTH = (uint8_t)(FLEXCAN_0.MB[0].MB_CS.B.LENGTH);
		/* Time stamp */
		g_can_RxData.TIME_STAMP = (uint16_t)(FLEXCAN_0.MB[0].MB_CS.B.TIME_STAMP);
		/* ID */
		g_can_RxData.ID = FLEXCAN_0.MB[0].MB_ID.B.ID;
		/* DATA */
		g_can_RxData.Data[0] = FLEXCAN_0.MB[0].MB_DATAL.B.DATA_BYTE_0;
		g_can_RxData.Data[1] = FLEXCAN_0.MB[0].MB_DATAL.B.DATA_BYTE_1;
		g_can_RxData.Data[2] = FLEXCAN_0.MB[0].MB_DATAL.B.DATA_BYTE_2;
		g_can_RxData.Data[3] = FLEXCAN_0.MB[0].MB_DATAL.B.DATA_BYTE_3;
		g_can_RxData.Data[4] = FLEXCAN_0.MB[0].MB_DATAH.B.DATA_BYTE_4;
		g_can_RxData.Data[5] = FLEXCAN_0.MB[0].MB_DATAH.B.DATA_BYTE_5;
		g_can_RxData.Data[6] = FLEXCAN_0.MB[0].MB_DATAH.B.DATA_BYTE_6;
		g_can_RxData.Data[7] = FLEXCAN_0.MB[0].MB_DATAH.B.DATA_BYTE_7;

		/* Clear INT flag, */
		FLEXCAN_0.IFLAG1.B.BUF5I = 0x1; /* The act of clearing the interrupt
		                                   triggers the FIFO engine to replace
		                                   the MB[0] with the next frame in the
		                                   queue, and then issue another
		                                   interrupt to the CPU. */
	}
}

/* ============================================================================
 *            CAN example API
 * ==========================================================================*/
/* ----------------------------------------------------------------------------
 * Name:        can_Init
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description: Initialization of M_CAN_1.
 *
 * --------------------------------------------------------------------------*/
void can_Init(void)
{ 
    uint32_t* p_memory;

    FLEXCAN_0.MCR.B.MDIS = 0; /* Enable the FlexCAN module */
    /* FlexCAN0 soft reset */
    FLEXCAN_0.MCR.B.SOFT_RST = 1U;
    /* Polls reset bit to know when the soft reset has completed. */
    while(FLEXCAN_0.MCR.B.SOFT_RST == 1U);

    /* Clear Shared CAN Message RAM */
    p_memory = (uint32_t*)(CAN_SHARED_RAM_ADD_START);
    while(p_memory < (uint32_t*)(CAN_SHARED_RAM_ADD_END))
    {
        *p_memory = 0;
        p_memory++;
    }

    /* Initialize the Module configuration register */
    FLEXCAN_0.MCR.B.FEN = 1; /* FIFO enabled */
    FLEXCAN_0.MCR.B.IDAM = 0; /* FIFO, One full ID (standard or extended) per
                                 filter element*/
    FLEXCAN_0.MCR.B.BCC = 1; /* Individual Rx masking and queue feature are
                                enabled*/
    /* Set individual mask register for ID filtering for FIFO, hits all IDs */
    FLEXCAN_0.RXIMR[0].R = 0;
    FLEXCAN_0.RXIMR[1].R = 0;
    FLEXCAN_0.RXIMR[2].R = 0;
    FLEXCAN_0.RXIMR[3].R = 0;
    FLEXCAN_0.RXIMR[4].R = 0;
    FLEXCAN_0.RXIMR[5].R = 0;
    FLEXCAN_0.RXIMR[6].R = 0;
    FLEXCAN_0.RXIMR[7].R = 0;

    /* Initialize the Control register */
    /* Baud rate initialization */
    FLEXCAN_0.CTRL.B.PRESDIV = CAN_CTRL_PRESDIV;
    FLEXCAN_0.CTRL.B.PSEG1 = CAN_CTRL_PSEG1;
    FLEXCAN_0.CTRL.B.PSEG2 = CAN_CTRL_PSEG2;
    FLEXCAN_0.CTRL.B.PROPSEG = CAN_CTRL_PROPSEG;
    FLEXCAN_0.CTRL.B.RJW = CAN_CTRL_RJW;

#if(CAN_LOOP_BACK_TEST)
    /* External Loop Back Test mode */
    FLEXCAN_0.CTRL.B.LPB = 1;
    FLEXCAN_0.MCR.B.SRX_DIS = 0; /* Self reception enabled */
#else
	FLEXCAN_0.CTRL.B.LPB = 0;
	FLEXCAN_0.MCR.B.SRX_DIS = 1; /* Self reception disabled */
#endif

    FLEXCAN_0.IMASK1.B.BUF5M = 1; /* MB5 completed reception or frames
                                     available in the FIFO. Interrupt
                                     is enabled */

	FLEXCAN_0.MB[0].MB_CS.R = 0x04000000; /* MB CODE: 0100b EMPTY: MB is active
	                                         and empty */

	FLEXCAN_0.MCR.B.MDIS = 0; /* Enable the FlexCAN module */
    FLEXCAN_0.MCR.B.HALT = 0; /* No Freeze Mode request */

    /* Register interrupt handler */
    bsp_isr_RegisterHandler(UC_CAN0_ISR_MB4_7, 4, CanIsrHandler);

    /* Configure FlexCAN0 RX and TX pins  */
	bsp_pin_EnableOutput(CAN_PIN_PC3, CAN_PIN_SSS_TX);   /* FlexCAN0_TX, PC3 */
	bsp_pin_EnableInput(CAN_PIN_PC2);                    /* FlexCAN0_RX, PC2 */
	/* FlexCAN0_RX functionality */
	SIUL2.MSCR_MUX[CAN_PIN_MUX_PC2].B.SSS = CAN_PIN_SSS_RX;
}

/* ----------------------------------------------------------------------------
 * Name:        can_SendData
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description:
 *              Function for frame send
 * --------------------------------------------------------------------------*/
void can_SendData(void)
{
	/* Clear Tx(INT) flag */
	FLEXCAN_0.IFLAG1.B.BUF8I = 0x1;

	FLEXCAN_0.MB[8].MB_CS.B.CODE = 0x8; /* MB: INACTIVE */
	FLEXCAN_0.MB[8].MB_ID.R = (uint32_t)(((uint32_t)(g_can_TxData.ID)) << 18);
	/* copy data to MB */
	FLEXCAN_0.MB[8].MB_DATAL.R =  \
			(uint32_t)((((uint32_t)(g_can_TxData.Data[0]))<<24)| \
			           (((uint32_t)(g_can_TxData.Data[1]))<<16)| \
					   (((uint32_t)(g_can_TxData.Data[2]))<<8) | \
					                g_can_TxData.Data[3]);

	FLEXCAN_0.MB[8].MB_DATAH.R =  \
			(uint32_t)((((uint32_t)(g_can_TxData.Data[4]))<<24)| \
			           (((uint32_t)(g_can_TxData.Data[5]))<<16)| \
			           (((uint32_t)(g_can_TxData.Data[6]))<<8) | \
						            g_can_TxData.Data[7]);

	/* Increments data[0] for next transmission */
 	g_can_TxData.Data[0] += 1;

 	/* CODE:Transmit data frame, LENGTH: */
	FLEXCAN_0.MB[8].MB_CS.R = \
			(uint32_t)((0xc << 24)|((g_can_TxData.LENGTH) << 16));
	FLEXCAN_0.MCR.B.HALT = 0;
}
/* End of file */
